@@ -255,8 +255,9 @@ static int fiemap_extent_process(__u32 clu, __u32 len, __u32 *rmap, __u32 rlen,
 		}
 
 		if (delta->l2_cache != l2_cluster) {
-			PREAD(delta, delta->l2, cluster,
-			      (off_t)l2_cluster * cluster);
+			if (PREAD(delta, delta->l2, cluster,
+						(off_t)l2_cluster * cluster))
+				return SYSEXIT_READ;
 			delta->l2_cache = l2_cluster;
 		}
 
@@ -266,19 +267,20 @@ static int fiemap_extent_process(__u32 clu, __u32 len, __u32 *rmap, __u32 rlen,
 			if (!delta->l2[j])
 				continue;
 
-			ridx = delta->l2[j] / B2S(cluster);
+			ridx = delta->l2[j] / ploop_sec_to_ioff(delta->blocksize,
+							delta->blocksize, delta->version);
 			if (ridx >= rlen) {
 				ploop_err(0,
-					"Image corrupted: L2[%u] == %u (max=%llu)",
+					"Image corrupted: L2[%u] == %u (max=%" PRIu64 ")",
 					clu + j - l2_slot, delta->l2[j],
-					(rlen - 1) * B2S(cluster));
+					(uint64_t)((rlen - 1) * B2S(cluster)));
 				return(SYSEXIT_PLOOPFMT);
 			}
 			if (ridx < delta->l1_size) {
 				ploop_err(0,
-					"Image corrupted: L2[%u] == %u (min=%llu)",
+					"Image corrupted: L2[%u] == %u (min=%" PRIu64 ")",
 					clu + j - l2_slot, delta->l2[j],
-					delta->l1_size * B2S(cluster));
+					(uint64_t)(delta->l1_size * B2S(cluster)));
 				return(SYSEXIT_PLOOPFMT);
 			}
 
@@ -530,20 +532,20 @@ static int range_build_rmap(__u32 iblk_start, __u32 iblk_end,
 				return SYSEXIT_READ;
 			delta->l2_cache = l2_cluster;
 		}
-
-		ridx = delta->l2[l2_slot] / B2S(cluster);
+		ridx = delta->l2[l2_slot] / ploop_sec_to_ioff(delta->blocksize,
+				delta->blocksize, delta->version);
 		if (ridx >= rlen) {
 			ploop_err(0,
-				"Image corrupted: L2[%u] == %u (max=%llu) (2)",
+				"Image corrupted: L2[%u] == %u (max=%" PRIu64 ") (2)",
 				clu, delta->l2[l2_slot],
-				(rlen - 1) * B2S(cluster));
+				(uint64_t)((rlen - 1) * B2S(cluster)));
 			return SYSEXIT_PLOOPFMT;
 		}
 		if (ridx && ridx < delta->l1_size) {
 			ploop_err(0,
-				"Image corrupted: L2[%u] == %u (min=%llu) (2)",
+				"Image corrupted: L2[%u] == %u (min=%" PRIu64 ") (2)",
 				clu, delta->l2[l2_slot],
-				delta->l1_size * B2S(cluster));
+				(uint64_t)(delta->l1_size * B2S(cluster)));
 			return SYSEXIT_PLOOPFMT;
 		}
 
@@ -701,7 +703,8 @@ static int range_split(struct freemap *rangemap, struct freemap *freemap,
 				l = MIN(ri + rl, fi) - ri;
 				ret = relocmap_add_extent(relocmap_pp, rc, ri, l, 0);
 				if (ret)
-					return ret;			}
+					return ret;
+			}
 
 			ri += l;
 			rc  += l;
@@ -735,8 +738,9 @@ int relocmap2relocblks(struct relocmap *relocmap, int lvl, __u32 a_h, __u32 n_sc
 	relocblks->alloc_head = a_h;
 	relocblks->n_scanned  = n_scanned;
 
-	for(i = 0; i < relocmap->n_entries_used; i++) {
+	for(i = 0; i < n; i++) {
 		if (!relocmap->extents[i].len) {
+			free(relocblks);
 			ploop_err(0, "abort: relocmap2relocblks !relocmap->extents[i].len");
 			return SYSEXIT_ABORT;
 		}
@@ -746,7 +750,7 @@ int relocmap2relocblks(struct relocmap *relocmap, int lvl, __u32 a_h, __u32 n_sc
 		relocblks->extents[i].free = relocmap->extents[i].free;
 	}
 
-	relocblks->n_extents = relocmap->n_entries_used;
+	relocblks->n_extents = n;
 	*relocblks_pp = relocblks;
 
 	return 0;
