@@ -301,6 +301,30 @@ int ploop_open_dd(struct ploop_disk_images_data **di, const char *fname)
 	return 0;
 }
 
+int find_image_idx_by_file(struct ploop_disk_images_data *di, const char *file)
+{
+	int i;
+	char basedir[PATH_MAX];
+	char image[PATH_MAX];
+
+	/* First we need to normalize the image file name
+	 * to be the same as in struct ploop_disk_images_data
+	 * as filled in by ploop_read_dd() and parse_xml().
+	 */
+	get_basedir(di->runtime->xml_fname, basedir, sizeof(basedir));
+	if (basedir[0] != 0 && file[0] != '/')
+		snprintf(image, sizeof(image), "%s%s", basedir, file);
+	else
+		snprintf(image, sizeof(image), "%s", file);
+
+	for (i = 0; i < di->nimages; i++) {
+		if (!strcmp(image, di->images[i]->file))
+			return i;
+	}
+
+	return -1;
+}
+
 int find_image_idx_by_guid(struct ploop_disk_images_data *di, const char *guid)
 {
 	int i;
@@ -346,17 +370,17 @@ static void remove_data_from_array(void **array, int nelem, int id)
 		array[i] = array[i+1];
 }
 
-int ploop_get_child_by_uuid(struct ploop_disk_images_data *di, const char *guid, char **child_guid)
+const char * ploop_find_child_by_guid(struct ploop_disk_images_data *di, const char *guid)
 {
 	int i;
 
 	for (i = 0; i < di->nsnapshots; i++) {
 		if (guidcmp(di->snapshots[i]->parent_guid, guid) == 0) {
-			*child_guid = di->snapshots[i]->guid;
-			return 0;
+			return di->snapshots[i]->guid;
 		}
 	}
-	return -1;
+
+	return NULL;
 }
 
 char *ploop_find_parent_by_guid(struct ploop_disk_images_data *di, const char *guid)
@@ -438,7 +462,7 @@ int ploop_di_remove_image(struct ploop_disk_images_data *di, const char *guid,
 	return 0;
 }
 
-int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid, char **fname)
+int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid)
 {
 	int i, snap_id, image_id, nr_ch;
 	struct ploop_image_data *image = NULL;
@@ -446,7 +470,7 @@ int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid, ch
 
 	snap_id = find_snapshot_by_guid(di, guid);
 	if (snap_id == -1) {
-		ploop_err(0, "Unable to find snapshot by uuid %s",
+		ploop_err(0, "Can't find snapshot by uuid %s",
 				guid);
 		return SYSEXIT_PARAM;
 	}
@@ -454,28 +478,23 @@ int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid, ch
 
 	image_id = find_image_idx_by_guid(di, guid);
 	if (image_id == -1) {
-		ploop_err(0, "Unable to find image by uuid %s",
+		ploop_err(0, "Can't find image by uuid %s",
 				guid);
 		return SYSEXIT_PARAM;
 	}
 	nr_ch = ploop_get_child_count_by_uuid(di, snapshot->parent_guid);
 	if (nr_ch > 1) {
-		ploop_err(0, "Unable to merge snapshot %s: "
+		ploop_err(0, "Can't merge to snapshot %s: "
 				"it has %d children",
-				guid, nr_ch);
+				snapshot->parent_guid, nr_ch);
 		return SYSEXIT_PARAM;
 	}
 	if (guidcmp(snapshot->parent_guid, NONE_UUID) == 0) {
-		ploop_err(0, "Unable to merge image %s: it is a base image",
-				guid);
+		ploop_err(0, "Can't merge snapshot %s: "
+				"it is a base image", guid);
 		return SYSEXIT_PARAM;
 	}
 	image = di->images[image_id];
-	if (fname != NULL) {
-		*fname = strdup(image->file);
-		if (*fname == NULL)
-			return SYSEXIT_MALLOC;
-	}
 
 	/* Caller passed child_guid S2 to delete S1 (S1 <- S2 <- S3) (S2 <- S3)
 	 * so it has merge S2 to S1 and we should update all S1 referrences to S2
@@ -503,3 +522,88 @@ int ploop_di_merge_image(struct ploop_disk_images_data *di, const char *guid, ch
 
 	return 0;
 }
+
+/* Do some trivial checks that snapshots between the two specified guids
+ * can be merged.
+ */
+int ploop_di_can_merge_images(struct ploop_disk_images_data *di,
+		const char *ancestor_guid, const char *descendant_guid)
+{
+	int anc_idx, dsc_idx, idx;
+	int found = 0;
+
+	/* Check that both guids specified exist */
+	anc_idx = find_snapshot_by_guid(di, ancestor_guid);
+	if (anc_idx == -1) {
+		ploop_err(0, "Can't find snapshot by uuid %s", ancestor_guid);
+		return SYSEXIT_PARAM;
+	}
+	dsc_idx = find_snapshot_by_guid(di, descendant_guid);
+	if (dsc_idx == -1) {
+		ploop_err(0, "Can't find snapshot by uuid %s", descendant_guid);
+		return SYSEXIT_PARAM;
+	}
+
+	/* Check for proper chain of snapshots in between, i.e.
+	 * descendant -> parent -> .... -> ancestor
+	 */
+	idx = dsc_idx;
+	while (!found) {
+		const char *p_guid;
+		int num_ch;
+
+		p_guid = di->snapshots[idx]->parent_guid;
+		if (guidcmp(p_guid, ancestor_guid) == 0)
+			found = 1;
+		else if (guidcmp(p_guid, NONE_UUID) == 0)
+			break;
+
+		idx = find_snapshot_by_guid(di, p_guid);
+		if (idx == -1) {
+			ploop_err(0, "Inconsistency detected: snapshot %s "
+					"can't be found", p_guid);
+			return SYSEXIT_DISKDESCR;
+		}
+
+		/* Check our parent don't have any other children */
+		num_ch = ploop_get_child_count_by_uuid(di, p_guid);
+		if (num_ch > 1) {
+			ploop_err(0, "Unable to merge to %s: "
+					"it has %d children",
+					p_guid, num_ch);
+			return SYSEXIT_PARAM;
+		}
+	}
+
+	if (!found) {
+		ploop_err(0, "Can't find %s to be an ancestor of %s",
+				ancestor_guid, descendant_guid);
+		return SYSEXIT_PARAM;
+	}
+
+	if (idx != anc_idx) { /* should never happen */
+		ploop_err(0, "Inconsistency: idx != anc_idx");
+		return SYSEXIT_SYS;
+	}
+
+	return 0;
+}
+
+/* Only call this function if ploop_di_can_merge_images() returned 0 */
+int ploop_di_merge_images(struct ploop_disk_images_data *di,
+		const char *guid, int depth)
+{
+	/* As guid is propagated to the previous level,
+	 * we just need to call di_merge_image repeatedly
+	 * using the same guid.
+	 */
+	while (depth--) {
+		int ret;
+
+		ret = ploop_di_merge_image(di, guid);
+		if (ret)
+			return ret;
+	}
+
+	return 0;
+}
