@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2008-2012, Parallels, Inc. All rights reserved.
+ *  Copyright (C) 2008-2015, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ static int do_delete_snapshot(struct ploop_disk_images_data *di, const char *gui
 	if (snap_id == -1) {
 		ploop_err(0, "Can't find snapshot by uuid %s",
 				guid);
-		return SYSEXIT_PARAM;
+		return SYSEXIT_NOSNAP;
 	}
 	ret = ploop_find_dev_by_dd(di, dev, sizeof(dev));
 	if (ret == -1)
@@ -102,15 +102,7 @@ static int do_delete_snapshot(struct ploop_disk_images_data *di, const char *gui
 			return SYSEXIT_PARAM;
 		}
 
-		char *parent_fname = find_image_by_guid(di, snap->parent_guid);
-		if (parent_fname == NULL) {
-			ploop_err(0, "Unable to find image by uuid %s",
-					snap->parent_guid);
-			return SYSEXIT_PARAM;
-		}
-
-		ret = check_snapshot_mount(di, snap->temporary, parent_fname,
-				fname, guid);
+		ret = check_snapshot_mount(di, guid, fname, snap->temporary);
 		if (ret)
 			return ret;
 
@@ -137,7 +129,7 @@ static int do_delete_snapshot(struct ploop_disk_images_data *di, const char *gui
 			ploop_log(0, "ploop snapshot %s has been successfully deleted",
 				guid);
 	} else if (nelem == 1) {
-		ret = ploop_merge_snapshot_by_guid(di, guid, PLOOP_MERGE_WITH_CHILD);
+		ret = ploop_merge_snapshot_by_guid(di, guid, PLOOP_MERGE_WITH_CHILD, NULL);
 	} else if (!di->snapshots[snap_id]->temporary) {
 		ploop_log(1, "Warning: Unable to delete snapshot %s as there are %d references"
 				" to it; marking it as temporary instead",
@@ -240,13 +232,13 @@ static int get_snapshot_count(struct ploop_disk_images_data *di)
 	if (images == NULL)
 		return -1;
 	n = get_list_size(images);
-	free_images_list(images);
+	ploop_free_array(images);
 
 	return n;
 }
 
 static int do_create_snapshot(struct ploop_disk_images_data *di,
-		const char *guid, int temporary)
+		const char *guid, const char *snap_dir, int temporary)
 {
 	int ret;
 	int fd;
@@ -325,8 +317,28 @@ static int do_create_snapshot(struct ploop_disk_images_data *di,
 	if (ret)
 		return ret;
 
-	snprintf(fname, sizeof(fname), "%s.%s",
-			di->images[0]->file, file_guid);
+	if (snap_dir != NULL) {
+		char *name;
+		char *dir;
+
+		dir = realpath(snap_dir, NULL);
+		if (dir == NULL) {
+			ploop_err(errno, "Error in realpath(%s)", snap_dir);
+			return SYSEXIT_CREAT;
+		}
+
+		name = strrchr(di->images[0]->file, '/');
+		if (name != NULL)
+			name++;
+		else
+			name = di->images[0]->file;
+		snprintf(fname, sizeof(fname), "%s/%s.%s",
+				dir, name,	file_guid);
+		free(dir);
+	} else
+		snprintf(fname, sizeof(fname), "%s.%s",
+				di->images[0]->file, file_guid);
+
 	ploop_di_change_guid(di, di->top_guid, snap_guid);
 	if (temporary)
 		ploop_di_set_temporary(di, snap_guid);
@@ -384,7 +396,7 @@ int ploop_create_snapshot(struct ploop_disk_images_data *di,
 	if (ploop_lock_dd(di))
 		return SYSEXIT_LOCK;
 
-	ret = do_create_snapshot(di, param->guid, 0);
+	ret = do_create_snapshot(di, param->guid, param->snap_dir, 0);
 
 	ploop_unlock_dd(di);
 
@@ -425,7 +437,7 @@ int ploop_create_temporary_snapshot(struct ploop_disk_images_data *di,
 	if (ploop_lock_dd(di))
 		return SYSEXIT_LOCK;
 
-	ret = do_create_snapshot(di, param->guid, 1);
+	ret = do_create_snapshot(di, param->guid, param->snap_dir, 1);
 	if (ret)
 		goto err_unlock;
 
@@ -461,7 +473,7 @@ err:
 	ploop_umount(mount_param.device, di);
 
 err_merge:
-	ploop_merge_snapshot_by_guid(di, param->guid, PLOOP_MERGE_WITH_CHILD);
+	ploop_merge_snapshot_by_guid(di, param->guid, PLOOP_MERGE_WITH_CHILD, NULL);
 
 err_unlock:
 	ploop_unlock_dd(di);
@@ -469,7 +481,7 @@ err_unlock:
 	return ret;
 }
 
-static int is_device_inuse(const char *dev)
+int is_device_inuse(const char *dev)
 {
 	int count;
 	char fname[PATH_MAX];
@@ -762,7 +774,7 @@ int ploop_switch_snapshot(struct ploop_disk_images_data *di, const char *guid, i
 {
 	struct ploop_snapshot_switch_param param = {};
 
-	param.guid = (char *) guid;
+	param.guid = guid;
 	param.flags = flags;
 
 	return ploop_switch_snapshot_ex(di, &param);
