@@ -86,12 +86,12 @@ int plooptool_grow(int argc, char **argv)
 		int ret;
 
 		struct ploop_disk_images_data *di;
-		ret = read_dd(&di, argv[0]);
+		ret = ploop_open_dd(&di, argv[0]);
 		if (ret)
 			return ret;
 
 		ret = ploop_grow_image(di, new_size);
-		ploop_free_diskdescriptor(di);
+		ploop_close_dd(di);
 
 		return ret;
 	}
