@@ -71,7 +71,7 @@ static void usage_init(void)
 {
 	fprintf(stderr,
 "Usage: ploop init -s SIZE [-f FORMAT] [-v VERSION] [-t FSTYPE]\n"
-"                 [-b BLOCKSIZE] [-B FSBLOCKSIZE] DELTA\n"
+"                 [-b BLOCKSIZE] [-B FSBLOCKSIZE] [-n|--nolazy] DELTA\n"
 "\n"
 "       SIZE        := NUMBER[KMGT]\n"
 "       FORMAT      := " USAGE_FORMATS "\n"
@@ -79,6 +79,7 @@ static void usage_init(void)
 "       FSTYPE      := { none | ext3 | ext4 } (create filesystem, default ext4)\n"
 "       BLOCKSIZE   := cluster block size, sectors\n"
 "       FSBLOCKSIZE := file system block size, bytes\n"
+"       -n, --nolazy - do not use lazy initialization during mkfs\n"
 "       DELTA       := path to a new image file\n"
 	);
 }
@@ -104,8 +105,14 @@ static int plooptool_init(int argc, char **argv)
 		.mode		= PLOOP_EXPANDED_MODE,
 		.fmt_version	= PLOOP_FMT_UNDEFINED,
 	};
+	static struct option long_opts[] = {
+		{ "nolazy", no_argument, 0, 'n' },
+		{},
+	};
 
-	while ((i = getopt(argc, argv, "s:b:B:f:t:v:")) != EOF) {
+
+	while ((i = getopt_long(argc, argv, "s:b:B:f:t:v:n",
+					long_opts, NULL)) != EOF) {
 		switch (i) {
 		case 's':
 			if (parse_size(optarg, &size_sec, "-s")) {
@@ -155,6 +162,9 @@ static int plooptool_init(int argc, char **argv)
 			}
 			param.fmt_version = f;
 			break;
+		case 'n':
+			param.flags |= PLOOP_CREATE_NOLAZY;
+			break;
 		default:
 			usage_init();
 			return SYSEXIT_PARAM;
@@ -727,7 +737,7 @@ static int plooptool_snapshot_switch(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	if ((argc != 1 && !is_xml_fname(argv[0])) || uuid == NULL) {
+	if (argc != 1 || !is_xml_fname(argv[0]) || uuid == NULL) {
 		usage_snapshot_switch();
 		return SYSEXIT_PARAM;
 	}
@@ -789,8 +799,9 @@ static int plooptool_snapshot_delete(int argc, char **argv)
 
 static void usage_snapshot_merge(void)
 {
-	fprintf(stderr, "Usage: ploop snapshot-merge [-u UUID | -A] [-n DELTA] DiskDescriptor.xml\n"
+	fprintf(stderr, "Usage: ploop snapshot-merge [-u UUID [-U UUID2] | -A] [-n DELTA] DiskDescriptor.xml\n"
 			"       -u UUID       snapshot to merge (top delta if not specified)\n"
+			"       -U UUID2      all snapshots in UUID..UUID2 range\n"
 			"       -n DELTA      new delta file to merge to\n");
 }
 
@@ -799,13 +810,18 @@ static int plooptool_snapshot_merge(int argc, char ** argv)
 	int i, ret;
 	struct ploop_merge_param param = {};
 
-	while ((i = getopt(argc, argv, "u:n:A")) != EOF) {
+	while ((i = getopt(argc, argv, "u:U:n:A")) != EOF) {
 		switch (i) {
 		case 'u':
 			param.guid = parse_uuid(optarg);
 			if (!param.guid)
 				return SYSEXIT_PARAM;
 			break;
+		case 'U':
+			param.guid2 = parse_uuid(optarg);
+			if (!param.guid2)
+				return SYSEXIT_PARAM;
+			break;
 		case 'A':
 			param.merge_all = 1;
 			break;
